@@ -253,8 +253,8 @@ idle(Sock, Pid) ->
 	%% simplistic version using the unnammed prepared statement and portal.
 	{equery, Ref, Pid, {Query, Params}} ->
 	    ParseP =    encode_message(parse, {"", Query, []}),
-	    BindP =     encode_message(bind,  {"", "", Params, [binary]}),
-	    DescribeP = encode_message(describe, {portal, ""}), 
+	    BindP =     encode_message(bind,  {"", "", Params, auto, [binary]}),
+	    DescribeP = encode_message(describe, {portal, ""}),
 	    ExecuteP =  encode_message(execute,  {"", 0}),
 	    SyncP =     encode_message(sync, []),
 	    ok = send(Sock, [ParseP, BindP, DescribeP, ExecuteP, SyncP]),
@@ -294,7 +294,7 @@ idle(Sock, Pid) ->
 	{execute, Ref, Pid, {Name, Params}} ->
 	    %%io:format("execute: ~p ~p ~n", [Name, Params]),
 	    begin % Issue first requests for the prepared statement.
-		BindP     = encode_message(bind, {"", Name, Params, [binary]}),
+		BindP     = encode_message(bind, {"", Name, Params, auto, [binary]}),
 		DescribeP = encode_message(describe, {portal, ""}),
 		ExecuteP  = encode_message(execute, {"", 0}),
 		FlushP    = encode_message(flush, []),
@@ -604,7 +604,7 @@ encode_message(bind, Bind={NamePortal, NamePrepared,
                                      fun (Bin) when is_binary(Bin) -> <<1:16/integer>>;
                                          (Text) -> <<0:16/integer>> end,
                                      Parameters),
-                {NParameters, erlang:concat_binary(ParamFormatsList)}
+                {NParameters, erlang:list_to_binary(ParamFormatsList)}
     end,
 
     ParametersList = lists:map(
@@ -625,14 +625,14 @@ encode_message(bind, Bind={NamePortal, NamePrepared,
                                <<Size:32/integer, Bin/binary>>
                        end,
                        Parameters),
-    ParametersP = erlang:concat_binary(ParametersList),
+    ParametersP = erlang:list_to_binary(ParametersList),
 
     NResultFormats = length(ResultFormats),
     ResultFormatsList = lists:map(
                           fun (binary) -> <<1:16/integer>>;
                               (text) ->   <<0:16/integer>> end,
                           ResultFormats),
-    ResultFormatsP = erlang:concat_binary(ResultFormatsList),
+    ResultFormatsP = erlang:list_to_binary(ResultFormatsList),
 
     %%io:format("encode bind: ~p~n", [{PortalP, PreparedP,
     %%			     NParameters, ParamFormatsP,
