@@ -10,6 +10,7 @@
 -compile(export_all).
 
 -include_lib("eunit/include/eunit.hrl").
+-include_lib("ts_macros.hrl").
 
 test()->
     ok.
@@ -21,6 +22,11 @@ add_time_test() ->
     New = ts_utils:add_time(Old,3),
     ?assertEqual(3*1000*1000, timer:now_diff(New, Old)).
 
+add_elapsed_test() ->
+    T1=?NOW,
+    T2=?NOW,
+    ?assert(ts_utils:elapsed(T1,T2) >= 0).
+
 node_to_hostname_test() ->
     ?assertEqual({ok, "foo"}, ts_utils:node_to_hostname('bar@foo')).
 
@@ -57,6 +63,43 @@ export_text_escape_test()->
     ?assertEqual("&quot;B&quot;",ts_utils:export_text("\"B\"")),
     ?assertEqual("&lt; A",ts_utils:export_text(<<"< A">>)).
 
+pack_test()->
+    Res = ts_utils:pack([node1,node1,node1,node3]),
+    ?assertEqual([[node1,node1,node1],[node3]], Res).
+
+pack_single_test()->
+    Res = ts_utils:pack([node1]),
+    ?assertEqual([[node1]], Res).
+
+pack_list_test()->
+    A=[a,a,a,a,b,c,c,d,d],
+    Res=[[a,a,a,a],[b],[c,c],[d,d]],
+    ?assertEqual(Res, ts_utils:pack(A)).
+
+pack_list2_test()->
+    A=[a,a,a,a,b,c,c,d,d,d],
+    Res=[[a,a,a,a],[b],[c,c],[d,d,d]],
+    ?assertEqual(Res, ts_utils:pack(A)).
+
+pack_list3_test()->
+    A=[a,a,a,a,b,c,c,d,d,d,d,d],
+    Res=[[a,a,a,a],[b],[c,c],[d,d,d,d,d]],
+    ?assertEqual(Res, ts_utils:pack(A)).
+
+pack_string_test()->
+    A=["a","a","a","a","b","c","c","d","d"],
+    Res=[["a","a","a","a"],["b"],["c","c"],["d","d"]],
+    ?assertEqual(Res, ts_utils:pack(A)).
+
+pack_dual_test()->
+    A=[a,b],
+    Res=[[a],[b]],
+    ?assertEqual(Res, ts_utils:pack(A)).
+
+ pack_singles_test()->
+    A=[a,b,c,d],
+    Res=[[a],[b],[c],[d]],
+    ?assertEqual(Res, ts_utils:pack(A)).
 
 pmap_test()->
     F = fun(X) ->X + 1 end,
@@ -82,12 +125,34 @@ pmapn_big_test()->
     ?assertEqual(ResP, Res).
 
 filtermap_test()->
-    Fun = fun(X) -> case X > 1 of 
-                        true -> {true, X + 1}; 
-                        _ -> false 
-                    end 
+    Fun = fun(X) -> case X > 1 of
+                        true -> {true, X + 1};
+                        _ -> false
+                    end
           end,
     ResP = ts_utils:filtermap(Fun, [1,2,3]),
     Res = [3, 4],
     ?assertEqual(ResP, Res).
 
+spread_list_test()->
+    A=[a,a,a,a,b,c,c,d],
+    Res=[a,b,c,d,a,c, a,a],
+    ?assertEqual(Res, ts_utils:spread_list(A)).
+
+spread_list2_test()->
+    A=[a,a,a,a,b,c,c,d,d],
+    Res=[a,b,c,d,a,c,d, a,a],
+    ?assertEqual(Res, ts_utils:spread_list(A)).
+
+spread_ids_test()->
+    A = [a,a,a,a,b,c,c,d],
+    SpreadedBeams = ts_utils:spread_list(A),
+    Id0 = 1,
+    {Res,LastId} = lists:mapfoldl(fun(A,Acc) -> {{A, Acc}, Acc+1} end, Id0, SpreadedBeams),
+    Expected = [{a,1},{b,2},{c,3},{d,4},{a,5},{c,6},{a,7},{a,8}],
+    ?assertEqual(Expected, Res).
+
+myset_env()->
+    myset_env(0).
+myset_env(N)->
+    application:set_env(stdlib, debug_level, N).
