@@ -21,7 +21,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 -module(ts_shell).
 -vc('$Id: ts_erlang.erl,v 0.0 2009/08/20 16:31:58 nniclaus Exp $ ').
@@ -33,8 +34,7 @@
 -include("ts_shell.hrl").
 -include_lib("kernel/include/file.hrl").
 
--export([init_dynparams/0,
-         add_dynparams/4,
+-export([add_dynparams/4,
          get_message/2,
          session_defaults/0,
          dump/2,
@@ -93,7 +93,7 @@ session_defaults() ->
 %% @doc Initialises the state for a new protocol session.
 %% @end
 new_session() ->
-    #shell{}.
+    #shell_sess{}.
 
 %% @spec decode_buffer(Buffer::binary(),Session::record(shell)) ->  NewBuffer::binary()
 %% @doc We need to decode buffer (remove chunks, decompress ...) for
@@ -102,24 +102,19 @@ new_session() ->
 decode_buffer(Buffer,#shell{}) ->
     Buffer. % nothing to do for shell
 
-%% @spec init_dynparams() -> dyndata()
-%% @doc Creates a new record/term for storing dynamic request data.
-%% @end
-init_dynparams() ->
-    #dyndata{proto=#shell_dyndata{}}.
 
 %% @spec add_dynparams(Subst, dyndata(), param(), hostdata()) -> {dyndata(), server()} | dyndata()
 %% Subst = term()
 %% @doc Updates the dynamic request data structure created by
 %% {@link ts_protocol:init_dynparams/0. init_dynparams/0}.
 %% @end
-add_dynparams(false, DynData, Param, HostData) ->
-    add_dynparams(DynData#dyndata.proto, Param, HostData);
-add_dynparams(true, DynData, Param, HostData) ->
-    NewParam = subst(Param, DynData#dyndata.dynvars),
-    add_dynparams(DynData#dyndata.proto,NewParam, HostData).
+add_dynparams(false, {_DynVars, Session}, Param, HostData) ->
+    add_dynparams(Session, Param, HostData);
+add_dynparams(true, {DynVars, Session}, Param, HostData) ->
+    NewParam = subst(Param, DynVars),
+    add_dynparams(Session,NewParam, HostData).
 
-add_dynparams(#shell_dyndata{}, Param, _HostData) ->
+add_dynparams(#shell_sess{}, Param, _HostData) ->
     Param.
 
 %%----------------------------------------------------------------------
