@@ -1,5 +1,5 @@
 %%%
-%%%  Copyright  IDEALX S.A.S. 2004
+%%%  Copyright © IDEALX S.A.S. 2004
 %%%
 %%%  Author : Nicolas Niclausse <nicolas.niclausse@niclux.org>
 %%%  Created: 20 Apr 2004 by Nicolas Niclausse <nicolas.niclausse@niclux.org>
@@ -22,7 +22,8 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 %%% common functions used by http clients to parse config
 
@@ -31,7 +32,7 @@
 -author('nicolas.niclausse@niclux.org').
 
 -export([parse_config/2, parse_URL/1, set_port/1, set_scheme/1,
-         check_user_agent_sum/1, set_query/1]).
+         check_user_agent_sum/1, set_query/1, encode_ipv6_address/1]).
 
 -include("ts_profile.hrl").
 -include("ts_http.hrl").
@@ -49,7 +50,7 @@
 parse_config(Element = #xmlElement{name=dyn_variable}, Conf = #config{}) ->
     ts_config:parse(Element,Conf);
 parse_config(Element = #xmlElement{name=http},
-             Config=#config{curid = Id, session_tab = Tab, servers = Servers,
+             Config=#config{curid = Id, session_tab = Tab, servers = [Server|_] = Servers,
                             sessions = [CurS | _], dynvar=DynVar,
                             subst    = SubstFlag, match=MatchRegExp}) ->
     Version  = ts_config:getAttr(string,Element#xmlElement.attributes, version, "1.1"),
@@ -77,7 +78,8 @@ parse_config(Element = #xmlElement{name=http},
                             version     = Version,
                             get_ims_date= Date,
                             content_type= ContentType,
-                            body        = Contents},
+                            body        = Contents,
+                            tag         = Config#config.tag},
     %% SOAP Support: Add SOAPAction header to the message
     Request2 = case lists:keysearch(soap,#xmlElement.name,
                                     Element#xmlElement.content) of
@@ -96,17 +98,73 @@ parse_config(Element = #xmlElement{name=http},
                             Request2#http_request.headers),
     Request3 = Request2#http_request{headers=Headers,cookie=Cookies},
     %% HTTP Authentication
-    Msg = case lists:keysearch(www_authenticate, #xmlElement.name,
+    Request4 = case lists:keysearch(www_authenticate, #xmlElement.name,
                                Element#xmlElement.content) of
               {value, AuthEl=#xmlElement{} } ->
                   UserId= ts_config:getAttr(string,AuthEl#xmlElement.attributes,
                                               userid, undefined),
+                  Type = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              type, "basic"),
+                  Nonce = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              nonce, undefined),
+                  Opaque = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              opaque, undefined),
+                  Cnonce = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                                cnonce, "%%ts_user_server:get_really_unique_id%%"),
+                  Nc = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              nc, "00000001"),
                   Passwd= ts_config:getAttr(string,AuthEl#xmlElement.attributes,
                                               passwd, undefined),
-                  NewReq=Request3#http_request{userid=UserId, passwd=Passwd},
+                  Realm = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              realm, undefined),
+                  QOP = ts_config:getAttr(string,AuthEl#xmlElement.attributes,
+                                              qop, undefined),
+                  ?LOGF("DIGEST ? : ~p ~p ~p", [Type, Nonce, Realm], ?WARN),
+                  Request3#http_request{userid=UserId, passwd=Passwd,
+                                        auth_type=Type, digest_nonce=Nonce,
+                                        digest_cnonce=Cnonce, digest_nc=Nc,
+                                        digest_opaque=Opaque, realm=Realm,
+                                        digest_qop=QOP};
+              _ ->
+                  Request3
+          end,
+    %% OAuth
+    Msg = case lists:keysearch(oauth, #xmlElement.name,
+                               Element#xmlElement.content) of
+              {value, AuthEl2=#xmlElement{} } ->
+                  ConsumerKey = ts_config:getAttr(string,AuthEl2#xmlElement.attributes,
+                                              consumer_key, undefined),
+                  ConsumerSecret = ts_config:getAttr(string,AuthEl2#xmlElement.attributes,
+                                              consumer_secret, undefined),
+                  AccessToken = ts_config:getAttr(string,AuthEl2#xmlElement.attributes,
+                                              access_token, []),
+                  AccessTokenSecret = ts_config:getAttr(string,AuthEl2#xmlElement.attributes,
+                                              access_token_secret, []),
+                  Encoding = ts_config:getAttr(string,AuthEl2#xmlElement.attributes,
+                                              encoding, "HMAC-SHA1"),
+                  AbsoluteURL = case URL of
+                    "http" ++ _Rest ->
+                      hd(string:tokens(URL, "?"));
+                    _Relative ->
+                      server_to_url(Server) ++ hd(string:tokens(URL, "?"))
+                  end,
+                  SigEncoding = case Encoding of
+                    "PLAINTEXT" ->
+                      plaintext;
+                    "HMAC-SHA1" ->
+                      hmac_sha1;
+                    "RSA-SHA1" ->
+                      rsa_sha1
+                  end,
+                NewReq=Request4#http_request{oauth_access_token=AccessToken,
+                                             oauth_url=AbsoluteURL,
+                                             oauth_access_secret=AccessTokenSecret,
+                                             oauth_consumer={ConsumerKey,
+                                                            ConsumerSecret,
+                                                            SigEncoding}},
                   set_msg(NewReq, {SubstFlag, MatchRegExp, UseProxy, Servers, PreviousHTTPServer, Tab, CurS#session.id} );
               _ ->
-                  set_msg(Request3, {SubstFlag, MatchRegExp, UseProxy, Servers, PreviousHTTPServer, Tab, CurS#session.id} )
+                  set_msg(Request4, {SubstFlag, MatchRegExp, UseProxy, Servers, PreviousHTTPServer, Tab, CurS#session.id} )
           end,
     ts_config:mark_prev_req(Id-1, Tab, CurS),
     ets:insert(Tab,{{CurS#session.id, Id},Msg#ts_request{endpage=true,
@@ -188,26 +246,34 @@ parse_cookie([_| Tail], Cookies) ->
 %% server. These informations are stored in the #ts_request record.
 set_msg(HTTP=#http_request{url="http" ++ URL},
         {SubstFlag, MatchRegExp, UseProxy, [Server|_], _PrevHTTPServer, Tab, Id}) ->
-    URLrec     = parse_URL("http" ++ URL),
-    Path       = set_query(URLrec),
-    HostHeader = set_host_header(URLrec),
-    Port       = set_port(URLrec),
-    Scheme     = set_scheme({URLrec#url.scheme,Server#server.type}),
-    ets:insert(Tab,{{http_server, Id}, {HostHeader, URLrec#url.scheme}}),
+    case {SubstFlag, re:run(URL, "%%.+%%")} of
+        {true, {match,_}} ->
+            %% url is a dynvar, don't preset host header
+            set_msg2(HTTP, #ts_request{ack = parse, subst = true, match = MatchRegExp });
+        _ ->
 
-    {RealServer, RealPath} = case UseProxy of
-                     true  -> {Server, "http"++ URL};
-                     false -> {#server{host=URLrec#url.host, port=Port, type=Scheme},
-                               Path}
-                 end,
+            URLrec     = parse_URL("http" ++ URL),
+            Path       = set_query(URLrec),
+            HostHeader = set_host_header(URLrec),
+            Port       = set_port(URLrec),
+            Scheme     = set_scheme({URLrec#url.scheme,Server#server.type}),
+            ets:insert(Tab,{{http_server, Id}, {HostHeader, URLrec#url.scheme}}),
 
-    set_msg2(HTTP#http_request{url=RealPath, host_header = HostHeader},
-             #ts_request{ack    = parse,
-                         subst  = SubstFlag,
-                         match  = MatchRegExp,
-                         host   = RealServer#server.host,
-                         scheme = RealServer#server.type,
-                         port   = RealServer#server.port});
+            {RealServer, RealPath} = case UseProxy of
+                                         true  ->
+                                             {Server, "http"++ URL};
+                                         false ->
+                                             {#server{host=URLrec#url.host,port=Port,type=Scheme},Path}
+                                     end,
+
+            set_msg2(HTTP#http_request{url=RealPath, host_header = HostHeader, use_proxy=UseProxy},
+                     #ts_request{ack    = parse,
+                                 subst  = SubstFlag,
+                                 match  = MatchRegExp,
+                                 host   = RealServer#server.host,
+                                 scheme = RealServer#server.type,
+                                 port   = RealServer#server.port})
+    end;
 
 %% relative URL, no previous HTTP server, use proxy, error !
 set_msg(_, {_, _, true, _Server, [],_Tab,_Id}) ->
@@ -219,7 +285,7 @@ set_msg(HTTP=#http_request{url="%%" ++ _TailURL},  {true, MatchRegExp, _Proxy, _
              #ts_request{ack = parse, subst = true, match = MatchRegExp });
 %% relative URL, no proxy, a single server => we can preset host header at configuration time
 set_msg(HTTPRequest, Args={_SubstFlag, _MatchRegExp, false, [Server], [],_Tab,_Id}) ->
-    ?LOG("Relative URL, single server ",?NOTICE),
+    ?LOG("Relative URL, single server ",?INFO),
     URL = server_to_url(Server) ++ HTTPRequest#http_request.url,
     set_msg(HTTPRequest#http_request{url=URL}, Args);
 %% relative URL, no proxy, several servers: don't set host header
@@ -241,9 +307,9 @@ set_msg(HTTPRequest, {SubstFlag, MatchRegExp, true, Server, {HostHeader, PrevSch
 %% Purpose: set param  in ts_request
 set_msg2(HTTPRequest, Msg) -> Msg#ts_request{ param = HTTPRequest }.
 
-server_to_url(#server{port=443, host= Host, type= Type}) when Type==ssl orelse Type==ssl6->
+server_to_url(#server{port=443, host= Host, type= Type}) when Type==ts_ssl orelse Type==ts_ssl6->
     "https://" ++ encode_ipv6_address(Host);
-server_to_url(#server{port=Port, host= Host, type= Type})when Type==ssl orelse Type==ssl6->
+server_to_url(#server{port=Port, host= Host, type= Type})when Type==ts_ssl orelse Type==ts_ssl6->
     "https://" ++ encode_ipv6_address(Host) ++ ":" ++ integer_to_list(Port);
 server_to_url(#server{port=80, host= Host})->
     "http://" ++ encode_ipv6_address(Host);
@@ -283,12 +349,12 @@ set_port(#url{port=Port}) -> integer_to_list(Port).
 %% server is configured with IPv6, we assume that the we should also
 %% use IPv6 for the given absolut URL
 %% @end
-set_scheme({http, gen_tcp6}) -> gen_tcp6;
-set_scheme({http, ssl6})     -> gen_tcp6;
-set_scheme({http, _})        -> gen_tcp;
-set_scheme({https, ssl6})    -> ssl6;
-set_scheme({https, gen_tcp6})-> ssl6;
-set_scheme({https, _})       -> ssl.
+set_scheme({http, ts_tcp6})  -> ts_tcp6;
+set_scheme({http, ts_ssl6})  -> ts_tcp6;
+set_scheme({http, _})        -> ts_tcp;
+set_scheme({https, ts_ssl6}) -> ts_ssl6;
+set_scheme({https, ts_tcp6}) -> ts_ssl6;
+set_scheme({https, _})       -> ts_ssl.
 
 
 set_query(URLrec = #url{querypart=""}) ->
