@@ -20,18 +20,24 @@
 %%%  In addition, as a special exception, you have the permission to
 %%%  link the code of this program with any library released under
 %%%  the EPL license and distribute linked combinations including
-%%%  the two.
+%%%  the two; the MPL (Mozilla Public License), which EPL (Erlang
+%%%  Public License) is based on, is included in this exception.
 
 -module(tsung_controller).
 -vc('$Id$ ').
 -author('nicolas.niclausse@niclux.org').
 
--export([start/2, start_phase/3, stop/1, stop_all/1, status/1]).
+-export([start/0, start/2, start_phase/3, stop/1, stop_all/1, status/1, status_str/0]).
 -behaviour(application).
 
--include("ts_profile.hrl").
+-include("ts_macros.hrl").
 -include_lib("kernel/include/file.hrl").
 
+
+%% start the application with it's dependencies
+start() ->
+    ts_utils:ensure_all_started(tsung_controller, permanent).
+
 %%----------------------------------------------------------------------
 %% Func: start/2
 %% Returns: {ok, Pid}        |
@@ -40,22 +46,33 @@
 %%----------------------------------------------------------------------
 start(_Type, _StartArgs) ->
     error_logger:tty(false),
-    {ok, {LogDir, _Name}} = ts_utils:setsubdir(?config(log_dir)),
-    erlang:display("Log directory is: " ++ LogDir),
-    LogFile = filename:join(LogDir, atom_to_list(node()) ++ ".log"),
-    case  error_logger:logfile({open, LogFile }) of
-        ok ->
-            case ts_controller_sup:start_link(LogDir) of
-                {ok, Pid} ->
-                    {ok, Pid};
-                Error ->
-                    io:format(standard_error,"Can't start ! ~p ~n",[Error]),
-                    Error
-            end;
-        {error, Reason} ->
-            Msg = "Error while opening log file: " ,
-            io:format(standard_error,Msg ++ " ~p ~n",[Reason]),
-            {error, Reason}
+    case ts_utils:setsubdir(?config(log_dir)) of
+        {error, {error, eacces} } ->
+            Msg = "Error while creating log directory in ~s: permission denied~n" ,
+            io:format(standard_error,Msg,[?config(log_dir)]),
+            halt(77);
+        {error, Err} ->
+            Msg = "Error while creating log directory : ~s~n" ,
+            io:format(standard_error,Msg,[Err]),
+            halt(1);
+        {ok, {LogDir, _Name}} ->
+            io:format(standard_io,"Log directory is: ~s~n", [LogDir]),
+            application:set_env(tsung_controller,log_dir_real,LogDir),
+            LogFile = filename:join(LogDir, atom_to_list(node()) ++ ".log"),
+            case  error_logger:logfile({open, LogFile }) of
+                ok ->
+                    case ts_controller_sup:start_link(LogDir) of
+                        {ok, Pid} ->
+                            {ok, Pid};
+                        Error ->
+                            io:format(standard_error,"Can't start ! ~p ~n",[Error]),
+                            halt(1)
+                    end;
+                {error, Reason} ->
+                    Msg = "Error while opening log file: " ,
+                    io:format(standard_error,Msg ++ " ~p ~n",[Reason]),
+                    halt(1)
+            end
     end.
 
 start_phase(load_config, _StartType, _PhaseArgs) ->
@@ -66,11 +83,11 @@ start_phase(load_config, _StartType, _PhaseArgs) ->
             File ->
                 T = case file:read_file_info(File) of
                         {ok, #file_info{size=Size}} when Size > 10000000 -> % > 10MB
-                            io:format(standard_error,"Can take up to 5mn to read config ~p~n ",[Size]),
-                            300000; % 10mn
+                            io:format(standard_error,"Can take up to 5mn to read config file of size ~p~n ",[Size]),
+                            300000; % 5mn
                         {ok, #file_info{size=Size}} when Size > 1000000 ->  % > 1MB
-                            io:format(standard_error,"Can take up to 3mn to read config ~p~n ",[Size]),
-                            180000; % 5mn
+                            io:format(standard_error,"Can take up to 3mn to read config file of size ~p~n ",[Size]),
+                            180000; % 3mn
                         {ok, #file_info{size=_}} ->
                             120000  % 2mn
                     end,
@@ -94,30 +111,39 @@ start_phase(start_clients, _StartType, _PhaseArgs) ->
 %%----------------------------------------------------------------------
 status([Host]) when is_atom(Host)->
     _List = net_adm:world_list([Host]),
-    global:sync(),
-    Msg = case catch ts_mon:status() of
-              {Clients, Count, Connected, Interval, Phase} ->
-
-                  S1 = io_lib:format("Tsung is running [OK]~n" ++
-                                     " Current request rate:    ~.2f req/sec~n" ++
-                                     " Current users:           ~p~n" ++
-                                     " Current connected users: ~p ~n",
-                                     [Count/Interval, Clients, Connected]),
-                  {ok, Nodes, Ended_Beams} = ts_config_server:status(),
-                  case {Phase, Nodes == Ended_Beams} of
-                      {error, _} -> % newphase not initialised, first phase
-                          S1 ++ " Current phase:           1";
-                      {_, true} ->
-                          S1 ++ " Current phase:           last, waiting for pending clients";
-                      {{ok,P}, _} ->
-                          NPhases = (P div Nodes) + 1,
-                          io_lib:format("~s Current phase:        ~p",[S1,NPhases])
-                  end;
-              {'EXIT', {noproc, _}} ->
-                  "Tsung is not started"
+    Msg = case global:sync() of
+              ok ->
+                  status_str();
+              {error, _Reason} ->
+                  "No status available: Can't synchronize with Tsung erlang nodes"
           end,
     io:format("~s~n",[Msg]).
 
+status_str()->
+    case catch ts_mon:status() of
+        {Clients, Count, Connected, Interval, Phase, _Cpu} ->
+
+            S1 = io_lib:format("Tsung is running [OK]~n" ++
+                                   " Current request rate:    ~.2f req/sec~n" ++
+                                   " Current users:           ~p~n" ++
+                                   " Current connected users: ~p ~n",
+                               [Count/Interval, Clients, Connected]),
+            {ok, Nodes, Ended_Beams, _} = ts_config_server:status(),
+            case {Phase, Nodes == Ended_Beams} of
+                {error, _} -> % newphase not initialised, first phase
+                    S1 ++ " Current phase:           1";
+                {_, true} ->
+                    S1 ++ " Current phase:           last, waiting for pending clients";
+                {{ok,P}, _} ->
+                    NPhases = (P div Nodes) + 1,
+                    io_lib:format("~s Current phase:        ~p",[S1,NPhases])
+            end;
+        {'EXIT', {noproc, _}} ->
+            "Tsung is not started";
+        {'EXIT', _Else} ->
+            "No status: Can't communicate with Tsung controller "
+    end.
+
 %%----------------------------------------------------------------------
 %% Func: stop/1
 %% Returns: any
